import 'dart:async';

import 'package:basf_logic_components/basf_logic_components.dart';
import 'package:basf_logic_components/domain/models/barcode_id.dart';
import 'package:basf_logic_components/scanner/validators/barcode_utils.dart';
import 'package:flutter/services.dart';

part 'scan_mapper.dart';

/// Returns a [Stream] of [BarcodeData] which includes latest scanned object.
/// Create an instance of a [HardwareScannerService] and then call
/// [initHardwareHandlerAndGetStream] to start listening to the [HardwareKeyboard]
/// then call [scanStream] to get the [Stream].
class HardwareScannerService {
  HardwareScannerService({
    required HardwareKeyboard hardwareKeyboard,
  }) : _hardwareKeyboard = hardwareKeyboard;

  final HardwareKeyboard _hardwareKeyboard;

  late final StreamController<String> _codeStreamController;
  late final StreamController<String> _barcodeStreamController;
  late final StreamSubscription<String> _codeStreamSubscription;

  String _scannedCode = '';

  Stream<String> initHardwareHandlerAndGetStream() {
    _codeStreamController = StreamController();
    _barcodeStreamController = StreamController();
    _hardwareKeyboard.addHandler(_handleKey);
    _codeStreamSubscription = _listenToCodeStream();
    return _barcodeStreamController.stream.asBroadcastStream();
  }

  bool _handleKey(KeyEvent event) {
    if (event is KeyDownEvent) {
      // Append GS character for key code 29
      // otherwise append the character directly
      _scannedCode += event.logicalKey.keyId == 29
          ? String.fromCharCode(29) : event.character
          ?? '';

      _codeStreamController.add(_scannedCode);
    }
    return false;
  }


  StreamSubscription<String> _listenToCodeStream() {
    return _codeStreamController.stream
        .debounceTime(Duration(milliseconds: 250))
        .listen(processFullCode);
  }

  void processFullCode(String code) {
    if (code.isNotEmpty) {
      _barcodeStreamController.add(_scannedCode);
      _scannedCode = ''; // Reset for next scan after sending
    }
  }

  void dispose() {
    _hardwareKeyboard.removeHandler(_handleKey);
    _codeStreamSubscription.cancel();
    _codeStreamController.close();
    _barcodeStreamController.close();
  }
}
